#include "precompile.h"

#include <a8/mutable_xobject.h>
#include <f8/msgqueue.h>
#include <f8/jsonhttprequest.h>

#include "handlermgr.h"

#include "GCListener.h"
#include "mastermgr.h"
#include "app.h"

#include "ss_proto.pb.h"

static void _GMOpsSelfChecking(std::shared_ptr<f8::JsonHttpRequest> request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    request->resp_xobj->SetVal("healthy", 1);
    request->resp_xobj->SetVal("max_rundelay", 10);
}

static void _GMOpsGetNodeId(std::shared_ptr<f8::JsonHttpRequest> request)
{
    request->resp_xobj->SetVal("errcode", 0);
    request->resp_xobj->SetVal("errmsg", "");
    request->resp_xobj->SetVal("node_id", App::Instance()->GetNodeId());
}

void HandlerMgr::Init()
{
    RegisterNetMsgHandlers();
    RegisterGMMsgHandler("Ops$selfChecking", _GMOpsSelfChecking);
    RegisterGMMsgHandler("Ops$getNodeId", _GMOpsGetNodeId);
    f8::MsgQueue::Instance()->RegisterCallBack
        (
         IM_ExecGM,
         [this] (const a8::Args& args)
         {
             int socket_handle = args.Get<int>(0);
             std::string url = args.Get<std::string>(1);
             std::string query_str = args.Get<std::string>(2);
             unsigned long saddr = args.Get<unsigned long>(3);
             ProcGMMsg(saddr, socket_handle, url, query_str);
         });
}

void HandlerMgr::UnInit()
{
}

void HandlerMgr::RegisterNetMsgHandlers()
{
    RegisterNetMsgHandler(&msmsghandler, &MasterMgr::_SS_MS_ResponseTargetServer);
}

void HandlerMgr::ProcGMMsg(unsigned long saddr, int sockhandle,
                           const std::string& url, const std::string& querystr)
{
    if (url != "/webapp/index.php") {
        GCListener::Instance()->SendText(sockhandle, a8::HttpResponse(404, ""));
        return;
    }

    a8::HTTPRequest request;
    a8::ParserUrlQueryString(querystr.c_str(), request);

    std::string msgname = a8::Get(request, "c").GetString() + "$" + a8::Get(request, "a").GetString();
    auto itr = gmhandlers_.find(msgname);
    if (itr != gmhandlers_.end()) {
        auto request = std::make_shared<f8::JsonHttpRequest>();
        request->saddr = saddr;
        request->socket_handle = sockhandle;
        request->query_str = querystr;
        request->params->ReadFromUrlQueryString(querystr);
        itr->second(request);

        if (!request->pending){
            std::string response;
            request->resp_xobj->ToJsonStr(response);
            GCListener::Instance()->SendText(sockhandle, a8::HttpResponse(response));
        }
    } else {
        GCListener::Instance()->SendText(sockhandle, a8::HttpResponse("{}"));
    }
}

void HandlerMgr::RegisterGMMsgHandler(const std::string& msgname,
                                      void (*handler)(std::shared_ptr<f8::JsonHttpRequest>))
{
    gmhandlers_[msgname] = handler;
}
