#pragma once

#include <a8/udpsession.h>
#include <a8/udplistener.h>

#include "ikcp.h"

class KcpSession : public a8::UdpSession
{
public:
    KcpSession();
    ~KcpSession();

    void Init(int socket_handle);
    void UnInit();
    void Update(long long tick);

    const sockaddr_in& GetAddr() const { return remote_addr_; }
    int GetSocketHandle() { return socket_handle_; }
    long long GetSecretKey() { return secret_key_; }
    void* GetSecretKeyDataPtr() { return &secret_key_; }

    void SendClientMsg(char* buf, int buf_len);
    virtual void OnRecvPacket(a8::UdpPacket* pkt) override;

    static int GetSecretKeyLen() { return sizeof(long long); }
    static long long ReadSecretKey(const char* buf, int buf_len)
    {
        return buf_len < GetSecretKeyLen() ? 0 : *((long long*)buf);
    }

protected:
    virtual void DecodeUserPacket(char* buf, int& offset, unsigned int buflen) override;

private:
    void UpdateInput();

private:
    int socket_handle_ = 0;
    long long secret_key_ = 0;
    ikcpcb* kcp_ = nullptr;

    long long init_tick_ = 0;
    sockaddr_in remote_addr_ = {};
};
